//
//  TwitterRESTEngine.m
//  SPTwitterEngine
//
//  Created by Sam Page on 30/08/11.
//  Copyright 2011 Sam Page. All rights reserved.
//

#import "TwitterRESTEngine.h"
#import "TwitterAuthEngine.h"

@implementation TwitterRESTEngine

- (id)initWithAccessToken:(OAToken *)_accessToken 
			   andRESTURL:(NSString *)_URL 
			 andURLParams:(NSString *)_URLParams 
			  andDelegate:(id)_delegate;
{
    self = [super init];
    if (self) {
		delegate = _delegate;
        accessToken = _accessToken;
		URL = _URL;
		URLParams = _URLParams;
		parser = [[SBJsonParser alloc] init];
		
		consumer = [[OAConsumer alloc] initWithKey:OAuthConsumerKey
											secret:OAuthConsumerSecret];
    }

    return self;
}

- (void)getData {
	
	request = [[OAMutableURLRequest alloc] initWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"%@%@", URL, URLParams]]
											  consumer:consumer
												 token:accessToken   // we don't have a Token yet
												 realm:nil   // our service provider doesn't specify a realm
									 signatureProvider:nil]; // use the default method, HMAC-SHA1
	
    [request setHTTPMethod:@"GET"];
	
	fetcher = [[OADataFetcher alloc] init];
	
    [fetcher fetchDataWithRequest:request
                         delegate:self
                didFinishSelector:@selector(download:didFinishWithData:)
                  didFailSelector:@selector(download:didFailWithError:)];
	
	[fetcher release];
}

- (void)download:(OAServiceTicket *)statusTicket didFinishWithData:(NSData *)data {
	
#ifdef DEBUG
	NSString *debugString = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
	NSLog(@"Data received: %@", debugString);
	[debugString release];
#endif
	
	if ([URL compare:HOME_TIMELINE] == NSOrderedSame)
		[self newTweetFromData:data];
	else if ([URL compare:USER_TIMELINE] == NSOrderedSame)
		[self newTweetFromData:data];
	else if ([URL compare:USER_DATA] == NSOrderedSame)
		[self handleUserProfileResponse:data];
	else if ([URL compare:FOLLOWERS] == NSOrderedSame)
		[self newUserListFromData:data];
	else if ([URL compare:FOLLOWING] == NSOrderedSame)
		[self newUserListFromData:data];
	else if ([URL compare:USER_LOOKUP] == NSOrderedSame)
		[self handleUserProfileResponse:data];
}

- (void)download:(OAServiceTicket *)statusTicket didFailWithError:(NSError *)error {

	NSLog(@"Data download failed: %@", error);
}

- (void)newTweetFromData:(NSData *)data {
	
	id response = [parser objectWithData:data];
	
	// The response will be a dictionary if an error has occured
	if ([response isKindOfClass:[NSDictionary class]]) {
		if ([response objectForKey:@"error"] != nil)
			[delegate performSelector:@selector(twitterRESTEngineDidFailWithError:) 
						   withObject:[NSError errorWithDomain:[response objectForKey:@"error"]
														  code:0 userInfo:nil]];
	}
	
	else {
		// The response is an array of dictionaries if all is well
		NSArray *statuses = [parser objectWithData:data];
		NSMutableArray *tweets = [[NSMutableArray alloc] init];
		
		for (NSDictionary *dict in statuses) {
			
			Tweet *tweet = [[Tweet alloc] init];

			if ([[dict objectForKey:@"user"] objectForKey:@"screen_name"] != nil)
				[tweet setUsername:[[dict objectForKey:@"user"] objectForKey:@"screen_name"]];
			if ([dict objectForKey:@"text"] != nil) // This will strip out annoying new lines
				[tweet setText:[[[dict objectForKey:@"text"] componentsSeparatedByCharactersInSet:[NSCharacterSet newlineCharacterSet]] componentsJoinedByString:@" "]];
			if ([[dict objectForKey:@"user"] objectForKey:@"name"] != nil)
				[tweet setRealname:[[dict objectForKey:@"user"] objectForKey:@"name"]];
			if ([[dict objectForKey:@"user"] objectForKey:@"profile_image_url"] != nil)
				[tweet setProfileImageURL:[[dict objectForKey:@"user"] objectForKey:@"profile_image_url"]];	
			if ([dict objectForKey:@"source"] != nil)
				[tweet setClient:[dict objectForKey:@"source"]];
			if ([dict objectForKey:@"created_at"] != nil)
				[tweet setTime:[dict objectForKey:@"created_at"]];
			
			if ([[dict objectForKey:@"geo"] isKindOfClass:[NSDictionary class]]) {
				[tweet setIsGeotagged:TRUE];
				[tweet setLatitude:[[[[dict objectForKey:@"geo"] objectForKey:@"coordinates"] objectAtIndex:0] floatValue]];
				[tweet setLongitude:[[[[dict objectForKey:@"geo"] objectForKey:@"coordinates"] objectAtIndex:1] floatValue]];
				CLLocationCoordinate2D coord = {.latitude = [tweet latitude], .longitude = [tweet longitude]};
				tweet.coordinate = coord;
			}
			[tweets addObject:tweet];
			[tweet release];
		}
		// Return the tweets back to the delegate caller
		[delegate performSelector:@selector(twitterRESTEngineDidReceiveTweets:) withObject:tweets];
		[tweets release];
	}
}

- (void)handleUserProfileResponse:(NSData *)data {
	
	id response = [parser objectWithData:data];
	
	if ([response isKindOfClass:[NSDictionary class]]) {
		if ([response objectForKey:@"error"] != nil) {
			[delegate performSelector:@selector(twitterRESTEngineDidFailWithError:) 
						   withObject:[NSError errorWithDomain:[response objectForKey:@"error"]
														  code:0 userInfo:nil]];
		}
		else
			[self newUserProfileFromDictionary:response];
	}
	
	else if ([response isKindOfClass:[NSArray class]]) {
		for (NSDictionary *userDict in response)
			[self newUserProfileFromDictionary:userDict];
	}
		
	
}

- (void)newUserProfileFromDictionary:(NSDictionary *)userDict {
	
	UserProfile *user = [[UserProfile alloc] init];
	
	if ([userDict objectForKey:@"screen_name"] != nil)
		[user setUsername:[userDict objectForKey:@"screen_name"]];
	if ([userDict objectForKey:@"name"] != nil)
		[user setRealName:[userDict objectForKey:@"name"]];
	if ([userDict objectForKey:@"created_at"] != nil)
		[user setMemberSince:[userDict objectForKey:@"created_at"]];
	if ([userDict objectForKey:@"profile_image_url"] != nil)
		[user setProfileImageURL:[userDict objectForKey:@"profile_image_url"]];
	if ([userDict objectForKey:@"location"] != nil)
		[user setLocation:[userDict objectForKey:@"location"]];
	if ([userDict objectForKey:@"id_str"] != nil)
		[user setUserID:[userDict objectForKey:@"id_str"]];
	if ([userDict objectForKey:@"url"] != nil)
		[user setWebsite:[userDict objectForKey:@"url"]];
	if ([userDict objectForKey:@"description"] != nil)
		[user setBio:[userDict objectForKey:@"description"]];
	if ([userDict objectForKey:@"statuses_count"] != nil)
		[user setStatusCount:[userDict objectForKey:@"statuses_count"]];
	if ([userDict objectForKey:@"followers_count"] != nil)
		[user setFollowCount:[userDict objectForKey:@"followers_count"]];
	if ([userDict objectForKey:@"friends_count"] != nil)
		[user setFollowingCount:[userDict objectForKey:@"friends_count"]];
	if ([[userDict objectForKey:@"status"] objectForKey:@"text"] != nil)
		[user setLatestStatus:[[userDict objectForKey:@"status"] objectForKey:@"text"]];
	
	[delegate twitterRESTEngineDidReceiveUserProfile:user];
	[user release];
}

- (void)newUserListFromData:(NSData *)data {
	
	// When we look up followers/following, we receive a list of user_ids that we then need to lookup to get their data
	id response = [parser objectWithData:data];
	
	if ([response isKindOfClass:[NSDictionary class]]) {
		
		NSDictionary *userListDict = [parser objectWithData:data];
		
		NSArray *userList = [userListDict objectForKey:@"ids"];
		
		NSMutableArray *firstHundredUserIDs = [[NSMutableArray alloc] init];
		
		for (int i = 0; i < 100 && i < [userList count]; i++)
			[firstHundredUserIDs addObject:[userList objectAtIndex:i]];
		
		[delegate twitterRESTEngineDidReceiveListOfUsers:firstHundredUserIDs];
		
		[firstHundredUserIDs release];
	}
}

@end
